/*******************************************************************
   This file is responsible for extracting TA facts from
   the appropriate cfx facts and writing the TA facts to the
   factbase file.
********************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "ExtractFacts.h"
#include "globalVars.h"
#include "common.h"

/****************************************
  static function declarations
*****************************************/

static void extract ( struct fileImportExportInfo **, int );
static void printWarningMsg ( int );

/*****************************************
   MACROS
******************************************/

#define PRINTREL(rel,subject,object) fprintf ( fd, "%s %s %s\n", rel, subject, object )

/****************************************
  static variable defintions
*****************************************/

static FILE *fd;

/*****************************************
   function definitons
******************************************/

static void printWarningMsg ( recordNum )
   int recordNum;
{
   fprintf (logFileNum, "WARNING: Could not extract facts from record %d .\n", recordNum );
}

static void extract ( fileInfo, numOfFiles )
   struct fileImportExportInfo **fileInfo;
   int numOfFiles;
{
   struct objectInfo *ptr;
   char currentFile[MAX_TOKEN_SIZE];
   int recordNum;
   BOOLEAN printRecordToLog = FALSE;
   char tmp[MAX_LINE_SIZE];
   int i;
   
   for ( i=0; i < numOfFiles + 1; i++ )
   {
      if ( fileInfo[i] )
      {
         if ( useLongFileName )
            fprintf (logFileNum, "\nExtracting facts from file: %s \n", fileInfo[i] -> longFileName );
         else
            fprintf (logFileNum, "\nExtracting facts from file: %s \n", fileInfo[i] -> fileName );
      
         ptr = fileInfo[i] -> fileRelRecords;
      
         if  ( useLongFileName )
            strncpy ( currentFile, fileInfo[i] -> longFileName, MAX_TOKEN_SIZE );
         else   
            strncpy ( currentFile, fileInfo[i] -> fileName, MAX_TOKEN_SIZE );
      
         recordNum = 0;
      
         while ( ptr )
         {
                 
                  recordNum++;
                  
                  switch ( ptr -> kind ) 
                  {
                     case EXPORT_PUBLIC_FCN_DEF :  
         /* 0 */            
                          if ( fileInfo [ atoi(ptr -> fileAttr) ] )
                          {
                              if ( useLongFileName )
                                 PRINTREL("funcdef", fileInfo[ atoi(ptr -> fileAttr) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL("funcdef", fileInfo[ atoi(ptr -> fileAttr) ] -> fileName, ptr -> itemName );
                                 
                              strcpy ( tmp, currentFile );
                              strcat ( tmp, ":" );
                              strcat ( tmp, ptr -> Subject_Line );
                              PRINTREL("defloc", ptr -> itemName, tmp );
                          }
                          else
                          {
                              printWarningMsg (recordNum); 
                              printRecordToLog = TRUE;
                          }
                          break;
                          
                     case EXPORT_PUBLIC_VAR_DEF :
         /* 1 */          
                          if ( fileInfo [ atoi(ptr -> fileAttr) ] )
                          {
                              if ( useLongFileName )
                                 PRINTREL ("vardef", fileInfo[ atoi(ptr -> fileAttr) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL ("vardef", fileInfo[ atoi(ptr -> fileAttr) ] -> fileName, ptr -> itemName );

                              strcpy ( tmp, currentFile );
                              strcat ( tmp, ":" );
                              strcat ( tmp, ptr -> Subject_Line );
                              PRINTREL ("vardefloc", ptr -> itemName, tmp );
                         
                          }
                          else
                          {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                          }
                          break;
                     
                     case EXPORT_STATIC_FCN_DEF :   
         /* 2 */            
                          if ( fileInfo [ atoi(ptr -> fileAttr) ] )
                          {    
                              if ( useLongFileName )
                                 PRINTREL ("funcdef", fileInfo[ atoi(ptr -> fileAttr) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL ("funcdef", fileInfo[ atoi(ptr -> fileAttr) ] -> fileName, ptr -> itemName );
                                                      
                              strcpy ( tmp, currentFile );
                              strcat ( tmp, ":" );
                              strcat ( tmp, ptr -> Subject_Line );
                              PRINTREL ("defloc", ptr -> itemName, tmp );
                          }                        
                          else
                          {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                          }
                          break;
                     
                     case EXPORT_STATIC_VAR_DEF :   
         /* 3 */                 
                          if ( fileInfo [ atoi(ptr -> fileAttr) ] ) 
                          {    
                              if ( useLongFileName )
                                 PRINTREL ("vardef", fileInfo[ atoi(ptr -> fileAttr) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL ("vardef", fileInfo[ atoi(ptr -> fileAttr) ] -> fileName, ptr -> itemName );
                                                                                  
                              strcpy ( tmp, currentFile );
                              strcat ( tmp, ":" );
                              strcat ( tmp, ptr -> Subject_Line );
                              PRINTREL ("vardefloc", ptr -> itemName, tmp );
                          }
                          else
                          {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                          }
                          break;
                          
                     case EXPORT_PUBLIC_FCN_DECL :
         /* 4 */
                          if ( fileInfo [ atoi(ptr -> Subject_File) ] )
                          {                              
                              if ( useLongFileName )
                                 PRINTREL ("funcdcl", fileInfo[ atoi(ptr -> Subject_File) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL ("funcdcl", fileInfo[ atoi(ptr -> Subject_File) ] -> fileName, ptr -> itemName );
                                                           
                              strcpy ( tmp, currentFile );
                              strcat ( tmp, ":" );
                              strcat ( tmp, ptr -> Subject_Line );
                              PRINTREL ("dclloc", ptr -> itemName, tmp );
                          }    
                          else
                          {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                          }
                          break;
         
                     case EXPORT_PUBLIC_VAR_DECL :
          /* 5 */           
                          if ( fileInfo [ atoi(ptr -> fileAttr) ] )
                          {    
                              if ( useLongFileName )
                                 PRINTREL ("vardcl", fileInfo[ atoi(ptr -> fileAttr) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL ("vardcl", fileInfo[ atoi(ptr -> fileAttr) ] -> fileName, ptr -> itemName );
                              
                              
                              strcpy ( tmp, currentFile );
                              strcat ( tmp, ":" );
                              strcat ( tmp, ptr -> Subject_Line );
                              PRINTREL ("vardclloc", ptr -> itemName, tmp );                              
                          }
                          else
                          {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                          }
                          break;
                          
                     case EXPORT_STATIC_FCN_DECL :
         /* 6 */  
                          if ( fileInfo[ atoi(ptr -> Subject_File) ] )
                          {    
                              if ( useLongFileName )
                                 PRINTREL ("funcdcl", fileInfo[ atoi(ptr -> Subject_File) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL ("funcdcl", fileInfo[ atoi(ptr -> Subject_File) ] -> fileName, ptr -> itemName );
                                                       
                              strcpy ( tmp, currentFile );
                              strcat ( tmp, ":" );
                              strcat ( tmp, ptr -> Subject_Line );
                              PRINTREL ("dclloc", ptr -> itemName, tmp );
                         }
                         else
                         {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                         }
                         break;
                     
                     case EXPORT_MACRO :
         /* 7 */                
                         if ( fileInfo[ atoi(ptr -> fileAttr) ] )
                         {
                              if ( useLongFileName )
                                 PRINTREL ("macrodef", fileInfo[ atoi(ptr -> fileAttr) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL ("macrodef", fileInfo[ atoi(ptr -> fileAttr) ] -> fileName, ptr -> itemName );
                                                       
                              strcpy ( tmp, currentFile );
                              strcat ( tmp, ":" );
                              strcat ( tmp, ptr -> Subject_Line );
                              PRINTREL ("macrodefloc", ptr -> itemName, tmp );
                         }     
                         else
                         {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                         }
                         break;
                     
                     case EXPORT_TYPE :
         /* 8 */                
                         /****
                           `typedcl` should be changed to `typedef`
                              
                              AND
                              
                           `typedclloc' should be changed to `typedefloc`
                         *****/
                         
                         if ( fileInfo[ atoi(ptr -> fileAttr) ] )
                         {
                              if ( useLongFileName )
                                 PRINTREL ("typedef", fileInfo[ atoi(ptr -> fileAttr) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL ("typedef", fileInfo[ atoi(ptr -> fileAttr) ] -> fileName, ptr -> itemName );
                         
                              strcpy ( tmp, currentFile );
                              strcat ( tmp, ":" );
                              strcat ( tmp, ptr -> Subject_Line );
                              PRINTREL ("typedefloc", ptr -> itemName, tmp );
                         }        
                         else
                         {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                         }
                         break;

                     case EXPORT_ENUM_TAG :
         /* 9 */                
                         /****
                           `enumdcl` should be changed to `enumdef`
                              
                              AND
                              
                           `enumdclloc' should be changed to `enumdefloc`
                         *****/
                         
                         if ( fileInfo[ atoi(ptr -> fileAttr) ] )
                         {    
                              if ( useLongFileName )
                                 PRINTREL ("enumdef", fileInfo[ atoi(ptr -> fileAttr) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL ("enumdef", fileInfo[ atoi(ptr -> fileAttr) ] -> fileName, ptr -> itemName );                                 
                              
                              strcpy ( tmp, currentFile );
                              strcat ( tmp, ":" );
                              strcat ( tmp, ptr -> Subject_Line );
                              PRINTREL ("enumdefloc", ptr -> itemName, tmp );
                         }                         
                         else
                         {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                         }
                         break;

                     case EXPORT_STRUCT :
         /* 11 */            
                         /****
                           `structdcl` should be changed to `structdef`
                              
                              AND
                              
                           `structdclloc' should be changed to `structdefloc`
                         *****/
                         
                         if ( fileInfo[ atoi(ptr -> fileAttr) ] )
                         {
                              if ( useLongFileName )
                                 PRINTREL ("structdef", fileInfo[ atoi(ptr -> fileAttr) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL ("structdef", fileInfo[ atoi(ptr -> fileAttr) ] -> fileName, ptr -> itemName );
                              
                              strcpy ( tmp, currentFile );
                              strcat ( tmp, ":" );
                              strcat ( tmp, ptr -> Subject_Line );
                              PRINTREL ("structdefloc", ptr -> itemName, tmp );
                         }          
                         else
                         {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                         }
                         break;
                     
                     case EXPORT_UNION :
         /* 12 */            
                         /****
                           `uniondcl` should be changed to `uniondef`
                              
                              AND
                              
                           `uniondclloc' should be changed to `uniondefloc`
                         *****/
                         
                         if ( fileInfo[ atoi(ptr -> fileAttr) ] )
                         {     
                              if ( useLongFileName )
                                 PRINTREL ("uniondef", fileInfo[ atoi(ptr -> fileAttr) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL ("uniondef", fileInfo[ atoi(ptr -> fileAttr) ] -> fileName, ptr -> itemName );
                              
                              strcpy ( tmp, currentFile );
                              strcat ( tmp, ":" );
                              strcat ( tmp, ptr -> Subject_Line );
                              PRINTREL ("uniondefloc", ptr -> itemName, tmp );
                         }
                         else
                         {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                         }
                         break;
                     
                     case INCLUDE_FILE :         
         /* 14 */            
                         if ( fileInfo[ atoi(ptr -> Subject_File) ] && 
                              fileInfo[ atoi(ptr -> Object_File) ] )
                         {
                              if ( useLongFileName )
                                 PRINTREL ("include", fileInfo[ atoi(ptr -> Subject_File) ] -> longFileName, fileInfo[ atoi(ptr -> Object_File) ] -> longFileName );
                              else
                                 PRINTREL ("include", fileInfo[ atoi(ptr -> Subject_File) ] -> longFileName, fileInfo[ atoi(ptr -> Object_File) ] -> fileName );
                              
                         }
                         else
                         {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                         }
                         break;
                     
                     case IMPORT_PUBLIC_FCN_DEF :
         /* 19 */
                         PRINTREL ("sourcecall", ptr -> scopeName, ptr -> itemName );
                         break;
                         
                     case IMPORT_PUBLIC_VAR_DEF:
         /* 20 */
                          PRINTREL ("sourceref", ptr -> scopeName, ptr -> itemName );
                          break;
                     
                     case IMPORT_STATIC_FCN_DEF :
         /* 23 */                 
                          PRINTREL ("sourcecall", ptr -> scopeName, ptr -> itemName );
                          break;
                                          
                     case IMPORT_STATIC_VAR_DEF:  
         /* 24 */
                          PRINTREL ("sourceref", ptr -> scopeName, ptr -> itemName );
                          break;
                     
                     case IMPORT_LINK_FCN :
         /* 25 */                 
                          PRINTREL ("linkcall", ptr -> scopeName, ptr -> itemName );
                          break;
                          
                     case IMPORT_LINK_VAR :
         /* 26 */               
                          PRINTREL ("linkref", ptr -> scopeName, ptr -> itemName );
                          break;
                     
                     case IMPORT_EXTERN_FCN :
         /* 27 */
                          PRINTREL ("librarycall", ptr -> scopeName, ptr -> itemName );
                          break;
                     
                     case IMPORT_MACRO :
         /* 30 */            
                         if ( fileInfo[ atoi(ptr -> Subject_File) ] ) 
                         {
                              if ( useLongFileName )
                                 PRINTREL ("usemacro", fileInfo[ atoi(ptr->Subject_File) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL ("usemacro", fileInfo[ atoi(ptr->Subject_File) ] -> fileName, ptr -> itemName );                                 
                         }
                         else
                         {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                         }
                         break;
                     
                     case IMPORT_TYPE :
         /* 31 */            
                         if ( fileInfo[ atoi(ptr -> Subject_File) ] ) 
                         {
                              if ( useLongFileName )                                  
                                 PRINTREL ("usetype", fileInfo[ atoi(ptr->Subject_File) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL ("usetype", fileInfo[ atoi(ptr->Subject_File) ] -> fileName, ptr -> itemName );                                 
                         }
                         else
                         {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                         }
                         break;
                     
                     case IMPORT_ENUM_TAG :
         /* 32 */            
                         if ( fileInfo[ atoi(ptr -> Subject_File) ] ) 
                         {
                              if ( useLongFileName )                                  
                                 PRINTREL ("useenum", fileInfo[ atoi(ptr->Subject_File) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL ("useenum", fileInfo[ atoi(ptr->Subject_File) ] -> fileName, ptr -> itemName );                                 
                         }
                         else
                         {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                         }
                         break;
                                                
         /* 33 */    case IMPORT_ENUM_ITEM :
         
                     /**** 
                        ideally I would like to output a tuple such as:
                           
                           useenumitem  proc itemName
                        
                        I have enough info to produce such a tuple however problems will occur when trying to 
                        induce higher level relationships as that info is not complete.
                        
                        For example I would need to know
                           1) what enum tag the enum item belongs to :
                           
                                enumlist enumTagName enumItem1
                                ...
                                ...
                                enumlist enumTagName enumItemN
                                
                                I do not have the info to do this
                        
                        
                        I can however output the following relationship :
                        
                                enumlist  fileName enumItem1
                                ..,
                                ...
                                enumlist  fileName enumItemN
                        
                        This is similar to 32 and 32 is a more clean relationship meaning that less
                        information is missing.
                                
                     *************/
                     
                     break;
                     
                     
                     case IMPORT_STRUCT :
         /* 34 */   
                         if ( fileInfo[ atoi(ptr -> Subject_File) ] ) 
                         {
                              if ( useLongFileName )                                  
                                 PRINTREL ("usestruct", fileInfo[ atoi(ptr->Subject_File) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL ("usestruct", fileInfo[ atoi(ptr->Subject_File) ] -> fileName, ptr -> itemName );                              
                         }
                         else
                         {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                         }
                         break;
                     
                     case IMPORT_UNION :
         /* 35 */   
                         if ( fileInfo[ atoi(ptr -> Subject_File) ] ) 
                         {
                              if ( useLongFileName )                                  
                                 PRINTREL ("useunion", fileInfo[ atoi(ptr->Subject_File) ] -> longFileName, ptr -> itemName );
                              else
                                 PRINTREL ("useunion", fileInfo[ atoi(ptr->Subject_File) ] -> fileName, ptr -> itemName );                              
                         }
                         else
                         {
                              printWarningMsg(recordNum);
                              printRecordToLog = TRUE;
                         }
                         break;

                     case EXPORT_PUBLIC_FCN_DEF_END :                   /*SS*/
         /* 36 */                                                       /*SS*/
                     case EXPORT_STATIC_FCN_DEF_END :                   /*SS*/
         /* 37 */                                                       /*SS*/
                          if ( fileInfo [ atoi(ptr -> fileAttr) ] )     /*SS*/
                          {                                             /*SS*/
                              strcpy ( tmp, currentFile );              /*SS*/
                              strcat ( tmp, ":" );                      /*SS*/
                              strcat ( tmp, ptr -> Subject_Line );      /*SS*/
                              PRINTREL("deflocend", ptr -> itemName, tmp);
                          }                                             /*SS*/
                          else                                          /*SS*/
                          {                                             /*SS*/
                              printWarningMsg (recordNum);              /*SS*/
                              printRecordToLog = TRUE;                  /*SS*/
                          }                                             /*SS*/
                          break;                                        /*SS*/
                                                                        /*SS*/
                     case EXPORT_ENUM_TAG_END :                         /*SS*/
         /* 38 */                                                       /*SS*/
                          if ( fileInfo [ atoi(ptr -> fileAttr) ] )     /*SS*/
                          {                                             /*SS*/
                              strcpy ( tmp, currentFile );              /*SS*/
                              strcat ( tmp, ":" );                      /*SS*/
                              strcat ( tmp, ptr -> Subject_Line );      /*SS*/
                              PRINTREL("enumdeflocend", ptr -> itemName, tmp );
                          }                                             /*SS*/
                          else                                          /*SS*/
                          {                                             /*SS*/
                              printWarningMsg (recordNum);              /*SS*/
                              printRecordToLog = TRUE;                  /*SS*/
                          }                                             /*SS*/
                          break;                                        /*SS*/
                                                                        /*SS*/
                     case EXPORT_STRUCT_END :                           /*SS*/
         /* 39 */                                                       /*SS*/
                          if ( fileInfo [ atoi(ptr -> fileAttr) ] )     /*SS*/
                          {                                             /*SS*/
                              strcpy ( tmp, currentFile );              /*SS*/
                              strcat ( tmp, ":" );                      /*SS*/
                              strcat ( tmp, ptr -> Subject_Line );      /*SS*/
                              PRINTREL("structdeflocend", ptr -> itemName, tmp );                          }                                             /*SS*/
                          else                                          /*SS*/
                          {                                             /*SS*/
                              printWarningMsg (recordNum);              /*SS*/
                              printRecordToLog = TRUE;                  /*SS*/
                          }                                             /*SS*/
                          break;                                        /*SS*/
                                                                        /*SS*/
                     case EXPORT_UNION_END :                            /*SS*/
         /* 40 */                                                       /*SS*/
                          if ( fileInfo [ atoi(ptr -> fileAttr) ] )     /*SS*/
                          {                                             /*SS*/
                              strcpy ( tmp, currentFile );              /*SS*/
                              strcat ( tmp, ":" );                      /*SS*/
                              strcat ( tmp, ptr -> Subject_Line );      /*SS*/
                              PRINTREL("uniondeflocend", ptr -> itemName, tmp );                          }                                             /*SS*/
                          else                                          /*SS*/
                          {                                             /*SS*/
                              printWarningMsg (recordNum);              /*SS*/
                              printRecordToLog = TRUE;                  /*SS*/
                          }                                             /*SS*/
                          break;                                        /*SS*/
                                                                        /*SS*/
                     default : 
                        
                        break;
                     
                  }
                  
                  if ( printRecordToLog )
                  {
                      fprintf ( logFileNum, "%d %-3s %-4s %-7s %-4s %-7s %-15s %s \n",
                                   ptr -> kind,
                                   ptr -> fileAttr,
                                   ptr -> Subject_File,
                                   ptr -> Subject_Line,
                                   ptr -> Object_File,
                                   ptr -> Object_Line,
                                   ptr -> scopeName,
                                   ptr -> itemName );
                      
                      printRecordToLog = FALSE;
                   
                  }
                  
                  ptr = ptr -> next;
                       
         } /* end while */
         
      } /* end if */
      
   } /* end for */

}   


void ExtractFacts ( fileInfo, numOfFiles, fdout )
   struct fileImportExportInfo **fileInfo;
   int numOfFiles;
   FILE *fdout;
{
   fd = fdout;
   extract ( fileInfo, numOfFiles );
}
