/*******************************************************************
   This file is responsible for parsing the ?.cfx.mr file and
   building the data structures that contain the import/export
   data.
*******************************************************************/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "ReadCfxData.h"
#include "globalProcs.h"
#include "globalVars.h"
#include "common.h"

/****************************************
  static function declarations
*****************************************/

static void loadFileTable ( int * );
static void getFileImportExportData ( struct fileImportExportInfo **, int );
static void printFilesThatContainImpExpTables ( struct fileImportExportInfo **, int );
static void initArray ( struct fileImportExportInfo **, int );

/****************************************
  static variable defintions
*****************************************/

static FILE *fd;
static int numOfFileImportExportTables = 0;

static const char *FILE_TABLE_HEADER = "# The File Table";

/*****************************************
   function definitions
******************************************/


static void loadFileTable ( numOfFiles )
   int *numOfFiles;
{
   /*********
     File Table is not required any more so no space is allocated for it.
     This routine just reads the file table from the .cfx.mr file but does
     not do anything with the data.
     
     The file information is read in later when the import/export data is
     read.  See function 'getFileImportExportData'.
   ***********/
   
   char line[MAX_LINE_SIZE];
   char token1[MAX_TOKEN_SIZE];
   
   while ( ! feof(fd) )
   {
      fgets ( line, MAX_LINE_SIZE ,fd );
      if ( strstr (line, FILE_TABLE_HEADER ) )
         break;
   }
   
   fscanf ( fd, "%d", numOfFiles );

   {
      int i;   
      for ( i=0; i < *numOfFiles; i++ )
      {
         
         fscanf ( fd, "%s", token1 );      
         fscanf ( fd, "%s", token1 );
      
      }  /* end for */
   } /* end scope */
}

static void getFileImportExportData ( fileInfo, numOfFiles )
   struct fileImportExportInfo **fileInfo;
   int numOfFiles;
{
   char token1[MAX_TOKEN_SIZE];
   struct objectInfo *p;
   int i,j;
   
   while ( j < numOfFiles || ! feof(fd) )
   {
      struct fileImportExportInfo *ptr = (struct fileImportExportInfo *) xmalloc ( sizeof (struct fileImportExportInfo) ); 
      
      j++;
      
      getToken ( token1, fd );
      ptr -> fileName = (char *) xmalloc ( strlen (token1) + 1 );
      strcpy ( ptr -> fileName, token1 );
      
      getToken ( token1, fd );
      ptr -> longFileName = (char *) xmalloc ( strlen (token1) + 1 );
      strcpy ( ptr -> longFileName, token1 );
      
      fscanf ( fd, "%d", &(ptr -> numOfFileRelRecords) );
      
      ptr -> fileRelRecords = (struct objectInfo *) xmalloc ( sizeof (struct objectInfo) );
      p = ptr -> fileRelRecords;
      
      for ( i=0; i < ptr -> numOfFileRelRecords; i++ )
      {
         fscanf ( fd, "%d", (int *) &(p -> kind) );
         
         fscanf ( fd, "%s", token1 );
         p -> fileAttr = (char *) xmalloc ( strlen (token1) + 1 );
         strcpy ( p -> fileAttr, token1 );
         
         
         fscanf ( fd, "%s", token1 );    
         p -> Subject_File = (char *) xmalloc ( strlen (token1) + 1 );
         strcpy ( p -> Subject_File, token1 );
         
         fscanf ( fd, "%s", token1);             
         p -> Subject_Line = (char *) xmalloc ( strlen (token1) + 1 );
         strcpy ( p -> Subject_Line, token1 );
         
         fscanf ( fd, "%s", token1 );    
         p -> Object_File = (char *) xmalloc ( strlen (token1) + 1 );
         strcpy ( p -> Object_File, token1 );
         
         fscanf ( fd, "%s", token1 );    
         p -> Object_Line = (char *) xmalloc ( strlen (token1) + 1 );
         strcpy ( p -> Object_Line, token1 );
         
         fscanf ( fd, "%s", token1 );         
         p -> scopeName = (char *) xmalloc ( strlen (token1) + 1 );
         strcpy ( p -> scopeName , token1 );
         
         fscanf ( fd, "%s", token1 );
         p -> itemName = (char *) xmalloc ( strlen (token1) + 1 );
         strcpy ( p -> itemName, token1 );
                  
         if ( i == 0 )
            fileInfo [ atoi (p->fileAttr) ] = ptr;
            
         if ( i != ptr -> numOfFileRelRecords - 1 )
         {
            p -> next = (struct objectInfo *) xmalloc ( sizeof(struct objectInfo) );
            p = p -> next;
         }
         else
         {
            p -> next = NULL;
         }
         
      }  /* end for */
      
      numOfFileImportExportTables += 1;
      
   }  /* end whild */
}


/***********************************************************************
Not all the files in the file table will have a import/export table.
Only the files that do have an import/export table will be listed here
as well as it's index in the 'fileInfo' array. 
************************************************************************/

static void printFilesThatContainImpExpTables ( fileInfo, numOfFiles )
  struct fileImportExportInfo **fileInfo;
  int numOfFiles;
{  
  int i;
  
  fprintf ( debugFileNum, "\n\nList of files that have import/export tables.\n" );
  fprintf ( debugFileNum,     "---------------------------------------------\n" );
  
  for ( i=0; i < numOfFiles; i++ ) 
  {
     if ( fileInfo[i] != NULL )
        fprintf ( debugFileNum ,"%d )  %-25s %s\n", i , fileInfo[i] -> fileName, fileInfo[i] -> longFileName );
  }
    
}

static void outputImpExpData ( fileInfo, numOfFiles )
   struct fileImportExportInfo **fileInfo;
   int numOfFiles;
{         
    struct objectInfo *p;
    int i;
     
    fprintf ( debugFileNum, "\n\nimport/export data\n" );
    fprintf ( debugFileNum,     "----------------------\n" );
    
    for ( i=0; i < numOfFiles; i++ )
    {   
       if ( fileInfo[i] != NULL )
       {    
           fprintf ( debugFileNum , "\n\n%s\n",  fileInfo[i] -> fileName);
           fprintf ( debugFileNum ,"-----------------------\n" );
           
           p = fileInfo[i] -> fileRelRecords;
           
           while ( p )
           {
              fprintf ( debugFileNum, "%-25s %d %s %s %s %s %s %s\n", p -> itemName,
                                                                      p -> kind,
                                                                      p-> fileAttr,
                                                                      p-> Subject_File,
                                                                      p-> Subject_Line,
                                                                      p-> Object_File,
                                                                      p-> Object_Line,
                                                                      p-> scopeName );
                             
              p = p -> next;
           
           }  /* end while */      
       
       } /* end if */
       
    } /* end for */   
}

static void initArray ( fileInfo, numOfFiles )
   struct fileImportExportInfo **fileInfo;
   int numOfFiles;
{
   int i;
   for ( i=0; i < numOfFiles + 1; i++ )
      fileInfo[i] = NULL;
}


void ReadCfxData ( fileInfo, numOfFiles, numOfFileTables, cfxfd )
   struct fileImportExportInfo ***fileInfo;
   int *numOfFiles;
   int *numOfFileTables;
   FILE *cfxfd;
{
   fd = cfxfd;
   
   loadFileTable ( numOfFiles );
   
   *fileInfo = (struct fileImportExportInfo **) xmalloc ( sizeof (void *) * ((*numOfFiles) + 1) );
   initArray ( *fileInfo, *numOfFiles );
   
   getFileImportExportData ( *fileInfo, *numOfFiles );
   
   *numOfFileTables = numOfFileImportExportTables;
   
   if ( debugMode )
   {
      printFilesThatContainImpExpTables ( *fileInfo, *numOfFiles );
      outputImpExpData ( *fileInfo, *numOfFiles );
   } 
}
