/****************************************************
macros used in multiple files
*****************************************************/

#ifndef COMMON
#define COMMON

typedef int BOOLEAN;

#define FALSE 0
#define TRUE 1

#define MAX_LINE_SIZE 4096
#define MAX_TOKEN_SIZE 1024

/* #define NUM_OF_FILES 1000  */

enum kindOfRelation 
{   
   EXPORT_PUBLIC_FCN_DEF,    /* definition of functions with external linkage */
   EXPORT_PUBLIC_VAR_DEF,    /* definition of variables with external linkage */

   /* Remaining exports for none-main files only */

   EXPORT_STATIC_FCN_DEF,    /* definition of functions with internal linkage */
   EXPORT_STATIC_VAR_DEF,    /* definition of variables with internal linkage */
   EXPORT_PUBLIC_FCN_DECL,   /* declaration of functions with external linkage */
   EXPORT_PUBLIC_VAR_DECL,   /* declaration of variables with external linkage */
   EXPORT_STATIC_FCN_DECL,   /* declaration of functions with internal linkage */
   EXPORT_MACRO,             /* macros (#define) */
   EXPORT_TYPE,              /* typdefs */
   EXPORT_ENUM_TAG,          /* enumerated type tag */
   EXPORT_ENUM_ITEM,         /* enumerated type item */
   EXPORT_STRUCT,            /* struct name */
   EXPORT_UNION,             /* union name */

   IMPORT_EXPORT_DIVIDER,

   INCLUDE_FILE,             /* file included (#include) */
   IMPORT_IMPLEMENT_PUBLIC_FCN,      /* a definition implements a declaration */
   IMPORT_IMPLEMENT_PUBLIC_VAR,      /* a definition implements a declaration */
   IMPORT_IMPLEMENT_STATIC_FCN,      /* a definition implements a declaration */
   IMPORT_IMPLEMENT_STATIC_VAR,      /* a definition implements a declaration */
   IMPORT_PUBLIC_FCN_DEF,    /* use a definition in another file */
   IMPORT_PUBLIC_VAR_DEF,    /* use a definition in another file */
   IMPORT_PUBLIC_FCN_DECL,   /* use a declaration in another file */
   IMPORT_PUBLIC_VAR_DECL,   /* use a declaration in another file */
   IMPORT_STATIC_FCN_DEF,    /* use a static definition in another file */
   IMPORT_STATIC_VAR_DEF,    /* use a static definition in another file */
   IMPORT_LINK_FCN,          /* link time importation of functions */
   IMPORT_LINK_VAR,          /* link time importation of variables */
   IMPORT_EXTERN_FCN,        /* fcn not defined in analyzed code */
   IMPORT_EXTERN_VAR,        /* var not defined in analyzed code */
   IMPORT_CODE_AND_VARS,     /* use code and local vars in another file */
   IMPORT_MACRO,             /* macros (#define) */
   IMPORT_TYPE,              /* typdefs */
   IMPORT_ENUM_TAG,          /* enumerated type tag */
   IMPORT_ENUM_ITEM,         /* enumerated type item */
   IMPORT_STRUCT,            /* struct name */
   IMPORT_UNION,             /* union name */

  /* the remaining five "relations" are for tagging the end of definitions */

   EXPORT_PUBLIC_FCN_DEF_END, /* end of EXPORT_PUBLIC_FCN_DEF */  /*SS*/
   EXPORT_STATIC_FCN_DEF_END, /* end of EXPORT_STATIC_FCN_DEF */  /*SS*/
   EXPORT_ENUM_TAG_END,       /* end of enumerated type tag */    /*SS*/
   EXPORT_STRUCT_END,         /* end of struct definition */      /*SS*/
   EXPORT_UNION_END,          /* end of union definition */       /*SS*/

   IMPORT_FCN_REF_TEMP,
   IMPORT_VAR_REF_TEMP,
   IMPORT_LINK_FCN_TEMP,
   IMPORT_LINK_VAR_TEMP,
   NO_KIND
};
   

struct objectInfo
{
   char *itemName;
   enum kindOfRelation kind;
   char *fileAttr;
   char *Subject_File;
   char *Subject_Line;
   char *Object_File;
   char *Object_Line;
   char *scopeName;
   struct objectInfo *next;
};

struct fileImportExportInfo
{
   char *fileName;
   char *longFileName;
   int numOfFileRelRecords;
   struct objectInfo *fileRelRecords;
};


#endif

