/*************************************************
   This is the main file for the lsadjust program.  
**************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "globalProcs.h"
#include "globalVars.h"
#include "common.h"

/********************************************
   macros
*********************************************/
#define MIN_ARGS 3
#define MAX_ARGS 5

/********************************************
   Static function declarations
*********************************************/
static void getProgramArgs(int argc, char **argv);
static void displayProgramSyntax (void);
static void writeProgramInfoToLog (void);
static void setup (void);
static void cleanUp ( struct fileImportExportInfo **, int );
static void freeMem ( struct objectInfo * );

/*********************************************
   Static variable declarations
**********************************************/
static char *VERSION = "0.3.6";
static char *DATE = "November 5, 1997";
static char *PROGRAM_NAME = "fbgen";

/*** File Names ****/
static char *inCfxMr;
static char *outRsfFactBase;

static char *logFile;
static char *debugFile;

/*** File descriptors ***/
static FILE *cfxMRfd;
static FILE *RSFactBasefd;


/*********************************************
   miscellaneous
**********************************************/

BOOLEAN cont = FALSE;

/*********************************************
   main
**********************************************/

void main( int argc, char **argv )
{
   struct fileImportExportInfo **fileInfo;
   int numOfFiles = 0;
   int numOfFileImportExportTables = 0;

   getProgramArgs (argc, argv);
   setup();
   writeProgramInfoToLog();
   fflush( logFileNum );
   
   ReadCfxData ( &fileInfo, &numOfFiles, &numOfFileImportExportTables,cfxMRfd );
   ExtractFacts ( fileInfo, numOfFiles, RSFactBasefd );
   
   cleanUp ( fileInfo, numOfFiles );
   free ( fileInfo );
   
   fclose ( cfxMRfd );
   fclose ( RSFactBasefd );
   fclose ( logFileNum );
   if ( debugMode )
      fclose ( debugFileNum );
}


static void freeMem ( fileRelRecord )
   struct objectInfo *fileRelRecord;
{
   if ( fileRelRecord )
   {
      freeMem ( fileRelRecord -> next );      
      free ( fileRelRecord );
   }
}

static void cleanUp ( fileInfo, numOfFiles )
   struct fileImportExportInfo **fileInfo;
   int numOfFiles;
{
   int i;
   
   for ( i=0; i < numOfFiles + 1;  i++ )
   {
      if ( fileInfo[i] )
         freeMem ( fileInfo[i] -> fileRelRecords );
   }

}

static void setup (void)
{ 
   logFile = (char *) xmalloc ( strlen(PROGRAM_NAME) + strlen(".log") + 1 );
   strcpy (logFile, PROGRAM_NAME);
   strcat ( logFile , ".log" );

   debugFile = (char *) xmalloc (strlen(PROGRAM_NAME) + strlen(".debug") + 1 );
   strcpy (debugFile, PROGRAM_NAME); 
   strcat ( debugFile , ".debug" );
   
   cfxMRfd = fopen ( inCfxMr, "r" );
   if ( !cfxMRfd )
   {
      printf ("\nCould not open file : %s for read.", inCfxMr );
      exit(1);
   }
   
   RSFactBasefd = fopen ( outRsfFactBase, "w" );
   if ( ! RSFactBasefd )
   {
      printf ("\nCould not open file : %s for write.", outRsfFactBase );
      exit(1);
   }
   
   logFileNum = fopen ( logFile, "w" );
   if ( !logFileNum )
   {
      printf ("\nCould not open file : %s for write.", logFile );
      exit(1);
   }
   
   if ( debugMode == TRUE ) 
   {
      debugFileNum = fopen ( debugFile, "w" );
      if ( !debugFileNum )
      {
         printf ("\nCould not open file : %s for write.", debugFile );
         exit(1);
      }
   }

} /* end setup */



static void getProgramArgs ( int argc, char **argv )
{
   char *argStr;
   int i = 1;

   if ( argc < MIN_ARGS || argc > MAX_ARGS )
   {
      printf ("\nIncorrect number of arguments.");
      displayProgramSyntax();
      cont = FALSE;
      exit(1);
   }
   else
   {
      while (1)
      {

          argStr = argv[i];
          if ( argStr == strchr( argStr, '-' ) )   /* '-' is first char in string
                                                           therefore legal switch     */
          {
             if ( strlen ( argStr ) >= 2 )
             {
                 switch ( argv[i][1] )
                 {
                               
                    case 'x':  debugMode = TRUE;
                               break;

                    case 'l':  useLongFileName = TRUE;
                               break;
                                   
                    default :  printf ("\nInvalid switch : %s", argStr);
                               displayProgramSyntax();
                               cont = FALSE;
                               exit(1);
    
                 } /* end switch */
             }  /* end if */

          }
          else
          {
          
             break;
          
          }  /* end if */ 

          i++;

      }  /* end while */

      inCfxMr = argv[i];
      outRsfFactBase = argv[i+1];

   }  /* end if */

}  /* end getProgramArgs */


static void displayProgramSyntax ( void )
{
   printf ("\n\n");
   printf ("Program usage:");
   printf ("\n");
   printf ("Version: %s   Date: %s", VERSION, DATE );
   printf ("\n\n");
   printf ("%s [options] <input cfx mr file> <output TA fact base file>", PROGRAM_NAME );
   printf ("\n\n");
   printf ("options:");
   printf ("\n\n");
      
   printf ("%-15s %s", "-l", "output long file names in fact base file" );
   printf ("\n\n");
   printf ("%-15s %s", "-x", "debug mode (creates a file called fbgen.debug)" );
   printf ("\n\n");
   
} /* end displayProgramSyntax */


static void writeProgramInfoToLog( void )
{
   fprintf ( logFileNum, "fbgen Program Environment Information\n");
   fprintf ( logFileNum, "----------------------------------------\n");
   
   if (cfxMRfd)
      fprintf ( logFileNum, "Input cfx mr file : %s \n", inCfxMr );
   
   if (RSFactBasefd)
      fprintf ( logFileNum, "Output fact base file : %s \n", outRsfFactBase );
   
   if (useLongFileName)
      fprintf ( logFileNum, "Long file names will be output.\n");
   else 
      fprintf ( logFileNum, "Short file names will be used.\n");

   if (logFileNum)
      fprintf ( logFileNum, "Log file : %s \n", logFile );
      
   if (debugFileNum)
      fprintf ( logFileNum, "Debug file : %s \n", debugFile );
   
   if (debugMode)
      fprintf ( logFileNum, "DebugMode on\n" );
      
} /* end writeProgramInfoToLog */

